/***************************************************************************
DESCRIPTION : Main controller
***************************************************************************/
`include "define.v"

module MainCntr
 (    
// TTCrx chip connections:  
 input TTC_BrcstStr, TTC_BCntRes,   TTC_CK40Des1,  TTC_EvCntRes,
       TTC_L1Accept, TTC_Ready,     TTC_DbErrStr,  TTC_SinErrStr,
 input [7:2] TTC_Brcst,
 //
 input [9:0] ET,   // for testing... 
// LocalBus connections:
 input LocCS_b, LocWRITE_b,
 input [`LB_Addr_Size-1:0] LocAddr,  // The Address bus.
 inout [`REG_WIDTH-1:0]   LocData,  // The bidir Data bus

// these ports connect to the rest of the FPGA
 input clk, clk2x, DLL_lock, L2IDLE_b, event_buffer_empty,
 input [1:0] L2StatusBits,
 input [2:0] UnlockCnt,
 input [8:1] SD, AFifoEmpty, AFifoFull, LW, 
 input [3:0] AFifo1Status, AFifo2Status, AFifo3Status, AFifo4Status, 
             AFifo5Status, AFifo6Status, AFifo7Status, AFifo8Status,   
 input [15:0] ErrAcc1, ErrAcc2, ErrAcc3, ErrAcc4, ErrAcc5, ErrAcc6, ErrAcc7 , ErrAcc8,
 input [`MAIN_DATA_WIDTH- 1 : 0]     L2Data,
 input [`SpyFIFO_WIDTH-1:0]    //  FE buses before the 2->3 demux
    InputSpyData1,  InputSpyData2,  InputSpyData3,  InputSpyData4,
    InputSpyData5,  InputSpyData6,  InputSpyData7,  InputSpyData8,
 input [`BCN_WIDTH-1:0]  BCNofFib1BZero, BCNofFib2BZero, BCNofFib3BZero, 
       BCNofFib4BZero, BCNofFib5BZero, BCNofFib6BZero, BCNofFib7BZero, BCNofFib8BZero,

 output AsyncFifoRst, Hard_rst, Soft_rst, Start, Stop, StRq,CalibTrig, 
        run_status, TTC_TestEnable, VME_TrigOut, LatTrim,  L1A, 
        TTC_Reset_b, bypass, CounterModeSel, ClockSelect,
        CompactNorm, CompactCalib, PatternMode,  TTC_BC0,  
 output LB_Tri,   //  for test only 
 output [2:0] LB_STATE, // state of FSM in LocBusSlave, for testing
 output [4:0] NS, NPS, //Programmable # of Samples and Pre-Samples (per L1A)
 output [8:1] TI_EN,  // individual enable for TLK2501
 output [7:0] LastTTCcommand,
 output [15:0] ProgInputDelay, 
 output [24:1] QieChMask,                          
 output [`REG_WIDTH-1:0] LB_DOUT,   //  for sending LocData    
 output [`EVN_WIDTH-1:0] Int_EvN, Int_CTrEvN,  // test only
 output [`OrN_WIDTH-1:0] OrN,
 output [`REG_WIDTH-1:0] HTRsubmodN, PipeLength,
 output [9:0] ETlow, EThigh,
 output [4:0] ETch_mux,
 output [17:0] 
          Threshold1,  Threshold2,  Threshold3,  Threshold4, 
          Threshold5,  Threshold6,  Threshold7,  Threshold8, 
          Threshold9,  Threshold10, Threshold11, Threshold12, 
          Threshold13, Threshold14, Threshold15, Threshold16, 
          Threshold17, Threshold18, Threshold19, Threshold20, 
          Threshold21, Threshold22, Threshold23, Threshold24, 
 output reg PatternSel,
 output reg [`EVN_WIDTH-1:0] EvN, CTrEvN,
 output reg [`BCN_WIDTH-1:0] BCN,  
   // LUT stuff
 input   [10:0]      
          InLUT_DOut1,  InLUT_DOut2,  InLUT_DOut3,  InLUT_DOut4,
          InLUT_DOut5,  InLUT_DOut6,  InLUT_DOut7,  InLUT_DOut8,
          InLUT_DOut9,  InLUT_DOut10, InLUT_DOut11, InLUT_DOut12,
          InLUT_DOut13, InLUT_DOut14, InLUT_DOut15, InLUT_DOut16,
          InLUT_DOut17, InLUT_DOut18, InLUT_DOut19, InLUT_DOut20,
          InLUT_DOut21, InLUT_DOut22, InLUT_DOut23, InLUT_DOut24,    
 output   reg  [6:0]     InLUT_Addr,
 output   reg  [10:0]    InLUT_Data,
 output  [24:1] InLUT_WE,  
 output  InLUT_Mode,    
 input   [7:0]       
          OutLUT_DOut1,  OutLUT_DOut2,  OutLUT_DOut3,  OutLUT_DOut4,
          OutLUT_DOut5,  OutLUT_DOut6,  OutLUT_DOut7,  OutLUT_DOut8,
          OutLUT_DOut9,  OutLUT_DOut10, OutLUT_DOut11, OutLUT_DOut12,
          OutLUT_DOut13, OutLUT_DOut14, OutLUT_DOut15, OutLUT_DOut16,
          OutLUT_DOut17, OutLUT_DOut18, OutLUT_DOut19, OutLUT_DOut20,
          OutLUT_DOut21, OutLUT_DOut22, OutLUT_DOut23, OutLUT_DOut24,      
 output   reg  [9:0]    OutLUT_Addr,
 output   reg  [7:0]    OutLUT_Data,
 output  [24:1] OutLUT_WE,  
 output  OutLUT_Mode, 
 output  [17:0] Pattern1, Pattern2, Pattern3, Pattern4,
                Pattern5, Pattern6, Pattern7, Pattern8    );
                   
//  ***  INTERNAL SIGNALS ***
 reg  L2OutSpyFIFOactive, TRAILER_d, TRAILER_2d, L2WrEn;
 reg  [1:0]  L2StatusBits_d;
 reg  [7:0] TTC_DbErrCnt, TTC_SinErrCnt;
 reg  [8:1] SD_d, SD_2d, SD_stretch;
 reg  [9:0] PatternsAddrA, PatternsAddrB;
 reg  [`BCN_WIDTH-1:0] BCN_d1, BCN_d2, BCN_d3, BCN_d4; 
 reg  [`EVN_WIDTH-1:0] EvN_d4, EvN_d3, EvN_d2, EvN_d1,
      CTrEvN_d4, CTrEvN_d3, CTrEvN_d2, CTrEvN_d1; 
 reg  [`MAIN_DATA_WIDTH- 1 : 0]  L2Data_d;
 wire TTC_L1A, TTC_Hard_rst, TTC_Soft_rst, TTC_Resync, 
      TTC_Start, TTC_Stop, TTC_StRq, TTC_AsyncFifoRst, BunchEr;
 wire VME_Hard_rst, VME_Soft_rst, VME_Start, VME_Stop,
      VME_StRq, VME_L1A, VME_AsyncFifoRst, VME_CalibTrig, VME_ResCTrEvN;
 wire L2OutSpyFIFOempty,  L2OutSpyFIFOfull;
 wire HEADER, TRAILER;
 wire InSpyFIFO1_RD,   InSpyFIFO2_RD,
      InSpyFIFO3_RD,   InSpyFIFO4_RD,
      InSpyFIFO5_RD,   InSpyFIFO6_RD,  
      InSpyFIFO7_RD,   InSpyFIFO8_RD,  
      L2OutSpyFIFO_RD;
 wire [`SpyFIFO_WIDTH-1:0]  L2OutSpyFIFO_out,
      InSpyFIFO1_out,    InSpyFIFO2_out,    
      InSpyFIFO3_out,    InSpyFIFO4_out,      
      InSpyFIFO5_out,    InSpyFIFO6_out,      
      InSpyFIFO7_out,    InSpyFIFO8_out;
 wire [8:1]  InSpyFIFO_empty,  InSpyFIFO_full,
             SnapRdEn, SnapFifoEmpty, PatternsWrEn;
 wire [15:0] TTC_test0, TTC_test1, TTC_test2, TTC_test3,
             TTC_test4, TTC_test5, TTC_test6, TTC_test7,
             TTC_test8, TTC_test9, TTC_testA, TTC_testB,
             TTC_testC, TTC_testD, TTC_testE, TTC_testF;
 wire [17:0] pad18_1, pad18_2, pad18_3, pad18_4,
             pad18_5, pad18_6, pad18_7, pad18_8;
 wire [17:0] Pattern1Back, Pattern2Back, Pattern3Back, Pattern4Back,
             Pattern5Back, Pattern6Back, Pattern7Back, Pattern8Back; 
 wire [35:0]  // only [17:0] will be used 
    SnapFIFO1_out, SnapFIFO2_out, SnapFIFO3_out, SnapFIFO4_out,
    SnapFIFO5_out, SnapFIFO6_out, SnapFIFO7_out, SnapFIFO8_out;
 wire [`BCN_WIDTH-1:0] Int_BCN,  MaxBCN, SetBCN; 
 wire [`REG_WIDTH-1:0] D_IN; 

//  *********  LOWER  LEVEL  MODULE  INSTANTIATION  *******
// TTCdecoder instantiation.    
// Format of port map is:  .InternalPortName(ext_port_expr)
  TTCdecoder TTCdecoder (
   .clk(clk),      .clk2x(clk2x),       .reset(VME_Hard_rst),
   .tTTC_BCntRes(TTC_BCntRes),          .TTC_Brcst(TTC_Brcst),
   .TTC_BrcstStr(TTC_BrcstStr),         .TTC_CK40Des1(TTC_CK40Des1),  
   .tTTC_EvCntRes(TTC_EvCntRes),        .TTC_L1Accept(TTC_L1Accept),
   .LastTTCcommand(LastTTCcommand),     .TTC_Resync(TTC_Resync),
   .TTC_Hard_rst(TTC_Hard_rst),         .TTC_Soft_rst(TTC_Soft_rst),  
   .TTC_Start(TTC_Start),               .TTC_Stop(TTC_Stop),
   .TTC_BC0(TTC_BC0),                   .TTC_StRq(TTC_StRq),  
   .TTC_CalibTrig(TTC_CalibTrig),       .VME_ResCTrEvN(VME_ResCTrEvN), 
   .TTC_AsyncFifoRst(TTC_AsyncFifoRst), .TTC_TestEnable(TTC_TestEnable),
   .TTC_test0(TTC_test0), .TTC_test1(TTC_test1), .TTC_test2(TTC_test2), 
   .TTC_test3(TTC_test3), .TTC_test4(TTC_test4), .TTC_test5(TTC_test5),
   .TTC_test6(TTC_test6), .TTC_test7(TTC_test7), .TTC_test8(TTC_test8),
   .TTC_test9(TTC_test9), .TTC_testA(TTC_testA), .TTC_testB(TTC_testB),
   .TTC_testC(TTC_testC), .TTC_testD(TTC_testD), .TTC_testE(TTC_testE),
   .TTC_testF(TTC_testF), .MaxBCN(MaxBCN), .BunchEr(BunchEr), .CTrEvN(Int_CTrEvN),
   .BCN(Int_BCN), .SetBCN(SetBCN),  .EvN(Int_EvN), .OrN(OrN),  .TTC_L1A(TTC_L1A) );

// LocBusSlave instantiation.
// Format of port map is:  .InternalPortName(ext_port_expr)
  wire[9:0] ETSpy;

  LocBusSlave  LocBusSlave  (
  .clk(clk),         .rst(Hard_rst),    .ClockSelect(ClockSelect),
  .LocCS_b(LocCS_b), .LocWRITE_b(LocWRITE_b), .LocAddr(LocAddr), .LocData(LocData),
  .DLL_lock(DLL_lock), .UnlockCnt(UnlockCnt), .event_buffer_empty(event_buffer_empty), 
  .CompactNorm(CompactNorm), .CompactCalib(CompactCalib),
  .run_status(run_status),     .TTC_Ready(TTC_Ready),
  .LastTTCcommand(LastTTCcommand),     .SD(SD_stretch), 
  .AFifoFull(AFifoFull),  .AFifoEmpty(AFifoEmpty),    .LW(LW),
  .AFifo1Status(AFifo1Status),    .AFifo2Status(AFifo2Status), 
  .AFifo3Status(AFifo3Status),    .AFifo4Status(AFifo4Status), 
  .AFifo5Status(AFifo5Status),    .AFifo6Status(AFifo6Status), 
  .AFifo7Status(AFifo7Status),    .AFifo8Status(AFifo8Status), 
  .VME_Hard_rst(VME_Hard_rst), .VME_Soft_rst(VME_Soft_rst), .VME_StRq(VME_StRq), 
  .VME_Start(VME_Start), .VME_Stop(VME_Stop),  .VME_AsyncFifoRst(VME_AsyncFifoRst),
  .VME_L1A(VME_L1A), .VME_CalibTrig(VME_CalibTrig),
  .VME_ResCTrEvN(VME_ResCTrEvN), .VME_TrigOut(VME_TrigOut),
  .RepeatMode(RepeatMode), .PatternMode(PatternMode),
  .GateTTCcommands(GateTTCcommands), .StartStop_Spy(StartStop_Spy),
  .CTrEvN(CTrEvN),      .SetBCN(SetBCN), 
  .ETlow(ETlow),  .EThigh(EThigh), .ETch_mux(ETch_mux),
  .ETSpy(ETSpy),.ETSpyFIFO_RD(ETSpyFIFO_RD),	.ETSpy_empty(ETSpy_empty),	.ETSpy_full(ETSpy_full),
  .TTC_Reset_b(TTC_Reset_b), .EvN(EvN), .MaxBCN(MaxBCN), .BunchEr(BunchEr),
  .TTC_DbErrCnt(TTC_DbErrCnt), .TTC_SinErrCnt(TTC_SinErrCnt),
  .ErrAcc1(ErrAcc1), .ErrAcc2(ErrAcc2), .ErrAcc3(ErrAcc3), .ErrAcc4(ErrAcc4),
  .ErrAcc5(ErrAcc5), .ErrAcc6(ErrAcc6), .ErrAcc7(ErrAcc7), .ErrAcc8(ErrAcc8),
  .InSpyFIFO_empty(InSpyFIFO_empty),  .InSpyFIFO_full(InSpyFIFO_full),
  .PatternsWrEn(PatternsWrEn),  .SnapRdEn(SnapRdEn),  
  .SnapFIFO1({SnapFifoEmpty[1], SnapFIFO1_out[17:0]}),
  .SnapFIFO2({SnapFifoEmpty[2], SnapFIFO2_out[17:0]}),
  .SnapFIFO3({SnapFifoEmpty[3], SnapFIFO3_out[17:0]}),
  .SnapFIFO4({SnapFifoEmpty[4], SnapFIFO4_out[17:0]}),
  .SnapFIFO5({SnapFifoEmpty[5], SnapFIFO5_out[17:0]}),
  .SnapFIFO6({SnapFifoEmpty[6], SnapFIFO6_out[17:0]}),
  .SnapFIFO7({SnapFifoEmpty[7], SnapFIFO7_out[17:0]}),
  .SnapFIFO8({SnapFifoEmpty[8], SnapFIFO8_out[17:0]}),  
  .TTC_test0(TTC_test0), .TTC_test1(TTC_test1), .TTC_test2(TTC_test2), 
  .TTC_test3(TTC_test3), .TTC_test4(TTC_test4), .TTC_test5(TTC_test5),
  .TTC_test6(TTC_test6), .TTC_test7(TTC_test7), .TTC_test8(TTC_test8),
  .TTC_test9(TTC_test9), .TTC_testA(TTC_testA), .TTC_testB(TTC_testB),
  .TTC_testC(TTC_testC), .TTC_testD(TTC_testD), .TTC_testE(TTC_testE),
  .TTC_testF(TTC_testF), .LatTrim(LatTrim),
  .L2OutSpyFIFOempty(L2OutSpyFIFOempty),    .L2OutSpyFIFOfull(L2OutSpyFIFOfull),
  .FE_link_ERR({InputSpyData8[17],InputSpyData7[17],InputSpyData6[17],InputSpyData5[17],
            InputSpyData4[17],InputSpyData3[17],InputSpyData2[17],InputSpyData1[17]}),
  .FE_link_DV({InputSpyData8[16],InputSpyData7[16],InputSpyData6[16],InputSpyData5[16],
            InputSpyData4[16],InputSpyData3[16],InputSpyData2[16],InputSpyData1[16]}),
  .L2OutSpyFIFO_out(L2OutSpyFIFO_out),    .L2OutSpyFIFO_RD(L2OutSpyFIFO_RD),
  .InSpyFIFO1_out(InSpyFIFO1_out),       .InSpyFIFO2_out(InSpyFIFO2_out),
  .InSpyFIFO3_out(InSpyFIFO3_out),       .InSpyFIFO4_out(InSpyFIFO4_out),
  .InSpyFIFO5_out(InSpyFIFO5_out),       .InSpyFIFO6_out(InSpyFIFO6_out),
  .InSpyFIFO7_out(InSpyFIFO7_out),       .InSpyFIFO8_out(InSpyFIFO8_out),
  .HTRsubmodN(HTRsubmodN),  .PipeLength(PipeLength),
  .NS(NS),    .NPS(NPS),   .TI_EN(TI_EN),   .QieChMask(QieChMask),
  .InSpyFIFO1_RD(InSpyFIFO1_RD),  .InSpyFIFO2_RD(InSpyFIFO2_RD),
  .InSpyFIFO3_RD(InSpyFIFO3_RD),  .InSpyFIFO4_RD(InSpyFIFO4_RD),
  .InSpyFIFO5_RD(InSpyFIFO5_RD),  .InSpyFIFO6_RD(InSpyFIFO6_RD),
  .InSpyFIFO7_RD(InSpyFIFO7_RD),  .InSpyFIFO8_RD(InSpyFIFO8_RD),
  .LB_STATE(LB_STATE),  .D_IN(D_IN),  .D_OUT(LB_DOUT),   .Tri(LB_Tri),
  .InLUT_DOut1(InLUT_DOut1),     .InLUT_DOut2(InLUT_DOut2),     .InLUT_DOut3(InLUT_DOut3), 
  .InLUT_DOut4(InLUT_DOut4),     .InLUT_DOut5(InLUT_DOut5),     .InLUT_DOut6(InLUT_DOut6), 
  .InLUT_DOut7(InLUT_DOut7),     .InLUT_DOut8(InLUT_DOut8),     .InLUT_DOut9(InLUT_DOut9), 
  .InLUT_DOut10(InLUT_DOut10),   .InLUT_DOut11(InLUT_DOut11),   .InLUT_DOut12(InLUT_DOut12), 
  .InLUT_DOut13(InLUT_DOut13),   .InLUT_DOut14(InLUT_DOut14),   .InLUT_DOut15(InLUT_DOut15), 
  .InLUT_DOut16(InLUT_DOut16),   .InLUT_DOut17(InLUT_DOut17),   .InLUT_DOut18(InLUT_DOut18), 
  .InLUT_DOut19(InLUT_DOut19),   .InLUT_DOut20(InLUT_DOut20),   .InLUT_DOut21(InLUT_DOut21), 
  .InLUT_DOut22(InLUT_DOut22),   .InLUT_DOut23(InLUT_DOut23),   .InLUT_DOut24(InLUT_DOut24), 
  .InLUT_WE(InLUT_WE),           .InLUT_Mode(InLUT_Mode),                              
  .OutLUT_DOut1(OutLUT_DOut1),   .OutLUT_DOut2(OutLUT_DOut2),   .OutLUT_DOut3(OutLUT_DOut3), 
  .OutLUT_DOut4(OutLUT_DOut4),   .OutLUT_DOut5(OutLUT_DOut5),   .OutLUT_DOut6(OutLUT_DOut6), 
  .OutLUT_DOut7(OutLUT_DOut7),   .OutLUT_DOut8(OutLUT_DOut8),   .OutLUT_DOut9(OutLUT_DOut9), 
  .OutLUT_DOut10(OutLUT_DOut10), .OutLUT_DOut11(OutLUT_DOut11), .OutLUT_DOut12(OutLUT_DOut12), 
  .OutLUT_DOut13(OutLUT_DOut13), .OutLUT_DOut14(OutLUT_DOut14), .OutLUT_DOut15(OutLUT_DOut15), 
  .OutLUT_DOut16(OutLUT_DOut16), .OutLUT_DOut17(OutLUT_DOut17), .OutLUT_DOut18(OutLUT_DOut18), 
  .OutLUT_DOut19(OutLUT_DOut19), .OutLUT_DOut20(OutLUT_DOut20), .OutLUT_DOut21(OutLUT_DOut21), 
  .OutLUT_DOut22(OutLUT_DOut22), .OutLUT_DOut23(OutLUT_DOut23), .OutLUT_DOut24(OutLUT_DOut24), 
  .OutLUT_WE(OutLUT_WE),         .OutLUT_Mode(OutLUT_Mode), 
  .Pattern1Back(Pattern1Back),   .Pattern2Back(Pattern2Back), 
  .Pattern3Back(Pattern3Back),   .Pattern4Back(Pattern4Back), 
  .Pattern5Back(Pattern5Back),   .Pattern6Back(Pattern6Back), 
  .Pattern7Back(Pattern7Back),   .Pattern8Back(Pattern8Back), 
  .bypass(bypass),   .CounterModeSel(CounterModeSel),  .ProgInputDelay(ProgInputDelay),
  .BCNofFib1BZero(BCNofFib1BZero), .BCNofFib2BZero(BCNofFib2BZero), 
  .BCNofFib3BZero(BCNofFib3BZero), .BCNofFib4BZero(BCNofFib4BZero), 
  .BCNofFib5BZero(BCNofFib5BZero), .BCNofFib6BZero(BCNofFib6BZero),
  .BCNofFib7BZero(BCNofFib7BZero), .BCNofFib8BZero(BCNofFib8BZero),
  .Threshold1(Threshold1),   .Threshold2(Threshold2),   .Threshold3(Threshold3),
  .Threshold4(Threshold4),   .Threshold5(Threshold5),   .Threshold6(Threshold6), 
  .Threshold7(Threshold7),   .Threshold8(Threshold8),   .Threshold9(Threshold9), 
  .Threshold10(Threshold10), .Threshold11(Threshold11), .Threshold12(Threshold12), 
  .Threshold13(Threshold13), .Threshold14(Threshold14), .Threshold15(Threshold15),  
  .Threshold16(Threshold16), .Threshold17(Threshold17), .Threshold18(Threshold18), 
  .Threshold19(Threshold19), .Threshold20(Threshold20), .Threshold21(Threshold21), 
  .Threshold22(Threshold22), .Threshold23(Threshold23), .Threshold24(Threshold24) 
          );   // close LocBusSlave instantiation
    
// commander instantiation.
// Format of port map is:  .InternalPortName(ext_port_expr)
 commander commander (  
  .clk(clk),       .TTC_Resync(TTC_Resync), 
  .L2IDLE_b(L2IDLE_b),  .GateTTCcommands(GateTTCcommands),
  .TTC_Hard_rst(TTC_Hard_rst),  .TTC_Soft_rst(TTC_Soft_rst),  .TTC_StRq(TTC_StRq),  
  .TTC_Start(TTC_Start), .TTC_Stop(TTC_Stop),   .TTC_CalibTrig(TTC_CalibTrig), 
  .TTC_L1A(TTC_L1A),  .TTC_AsyncFifoRst(TTC_AsyncFifoRst),
  .VME_Hard_rst(VME_Hard_rst),  .VME_Soft_rst(VME_Soft_rst),  .VME_StRq(VME_StRq),
  .VME_Start(VME_Start), .VME_Stop(VME_Stop), .VME_AsyncFifoRst(VME_AsyncFifoRst),
  .VME_L1A(VME_L1A), .VME_CalibTrig(VME_CalibTrig),   
  .Hard_rst(Hard_rst),  .Soft_rst(Soft_rst),   .StRq(StRq),   .CalibTrig(CalibTrig),
  .Start(Start), .Stop(Stop),  .L1A(L1A), .AsyncFifoRst(AsyncFifoRst),
  .run_status(run_status)  );

 always @ (posedge clk) begin 
   // registers without reset: just delays and registration
      InLUT_Data    <= D_IN[10:0];
      InLUT_Addr    <= LocAddr[6:0];  
      OutLUT_Data   <= D_IN[7:0];
      OutLUT_Addr   <= LocAddr[9:0]; 
      PatternsAddrA <= LocAddr[9:0]; // addressing inside one PatternsRAM. 
      SD_d <= SD;  SD_2d <= SD_d;  SD_stretch <= SD & SD_d & SD_2d; // stretched optical SD
      // delaying BCN and EvN to match delay of L1A in commander.v
      BCN <=  BCN_d4;      BCN_d4 <=  BCN_d3;   BCN_d3 <=  BCN_d2;
      BCN_d2  <=  BCN_d1;  BCN_d1 <=  Int_BCN;  
      EvN    <=  EvN_d4;  EvN_d4 <= EvN_d3;   EvN_d3 <= EvN_d2;
      EvN_d2 <=  EvN_d1;  EvN_d1 <= Int_EvN;  
      CTrEvN    <=  CTrEvN_d4;  CTrEvN_d4 <= CTrEvN_d3;  CTrEvN_d3 <= CTrEvN_d2;
      CTrEvN_d2 <=  CTrEvN_d1;  CTrEvN_d1 <= Int_CTrEvN;  
  if (Hard_rst) begin  // reset  
      TTC_DbErrCnt <= 0;   TTC_SinErrCnt <= 0;    
  end   // reset
  else begin      // counting TTC errors  
   if (TTC_DbErrStr & (TTC_DbErrCnt != 8'hFF) ) TTC_DbErrCnt <= TTC_DbErrCnt + 1;
   if (TTC_SinErrStr& (TTC_SinErrCnt != 8'hFF)) TTC_SinErrCnt<= TTC_SinErrCnt+ 1;
  end  // else
 end // always


// ************************ TESTABILITY MODULES **************************
// *** 8 Test Pattern RAMs:  port A <--> LocalBus;  port B --> DataPath ******
 
reg pstate;
parameter WAIT=0, GO=1;
wire goram = PatternMode && TTC_TestEnable;
wire lastadd = (PatternsAddrB == 10'h3FF);
wire stopram = RepeatMode ? ~PatternMode : lastadd;

always @ (posedge clk2x or posedge Hard_rst)
   if (Hard_rst) begin
     PatternSel <= 0;
     PatternsAddrB <= 0;
     pstate <= WAIT;
   end
   else begin
     case (pstate)
       WAIT: begin
         PatternSel <= 0;
         PatternsAddrB <= 0;
         if (goram) pstate <= GO;
         else pstate <= WAIT;
         end
       GO: begin
         PatternSel <= 1;
         PatternsAddrB <= PatternsAddrB + 1;
         if (stopram) pstate <= WAIT;
         else pstate <= GO;
       end
     endcase
   end // else

 RAMB16_S18_S18 PatternRAM1 (
      .CLKA(clk),     .ENA(1'b1),    .WEA(PatternsWrEn[1]),   // PORT A 
      .SSRA(Hard_rst),              // Synchronous Set/Reset Input
      .ADDRA(PatternsAddrA),  
      .DIA(D_IN[15:0]),            .DIPA(D_IN[17:16]),   
      .DOA(Pattern1Back[15:0]),    .DOPA(Pattern1Back[17:16]),   
      .CLKB(clk2x),   .ENB(PatternSel),    .WEB(1'b0),        // PORT B 
      .SSRB(Hard_rst),              // Synchronous Set/Reset Input
      .ADDRB(PatternsAddrB),   .DIB(),       .DIPB(),   
      .DOB(Pattern1[15:0]),    .DOPB(Pattern1[17:16])            );
 RAMB16_S18_S18 PatternRAM2 (
      .CLKA(clk),     .ENA(1'b1),    .WEA(PatternsWrEn[2]),   // PORT A 
      .SSRA(Hard_rst),              // Synchronous Set/Reset Input
      .ADDRA(PatternsAddrA),  
      .DIA(D_IN[15:0]),            .DIPA(D_IN[17:16]),   
      .DOA(Pattern2Back[15:0]),    .DOPA(Pattern2Back[17:16]),   
      .CLKB(clk2x),   .ENB(PatternSel),    .WEB(1'b0),        // PORT B 
      .SSRB(Hard_rst),              // Synchronous Set/Reset Input
      .ADDRB(PatternsAddrB),   .DIB(),       .DIPB(),  
      .DOB(Pattern2[15:0]),    .DOPB(Pattern2[17:16])            );
 RAMB16_S18_S18 PatternRAM3 (
      .CLKA(clk),     .ENA(1'b1),    .WEA(PatternsWrEn[3]),   // PORT A 
      .SSRA(Hard_rst),              // Synchronous Set/Reset Input
      .ADDRA(PatternsAddrA),  
      .DIA(D_IN[15:0]),            .DIPA(D_IN[17:16]),   
      .DOA(Pattern3Back[15:0]),    .DOPA(Pattern3Back[17:16]),   
      .CLKB(clk2x),   .ENB(PatternSel),    .WEB(1'b0),        // PORT B 
      .SSRB(Hard_rst),              // Synchronous Set/Reset Input
      .ADDRB(PatternsAddrB),   .DIB(),       .DIPB(),   
      .DOB(Pattern3[15:0]),    .DOPB(Pattern3[17:16])            );
 RAMB16_S18_S18 PatternRAM4 (
      .CLKA(clk),     .ENA(1'b1),    .WEA(PatternsWrEn[4]),   // PORT A 
      .SSRA(Hard_rst),              // Synchronous Set/Reset Input
      .ADDRA(PatternsAddrA),  
      .DIA(D_IN[15:0]),            .DIPA(D_IN[17:16]),   
      .DOA(Pattern4Back[15:0]),    .DOPA(Pattern4Back[17:16]),   
      .CLKB(clk2x),   .ENB(PatternSel),    .WEB(1'b0),        // PORT B 
      .SSRB(Hard_rst),              // Synchronous Set/Reset Input
      .ADDRB(PatternsAddrB),   .DIB(),       .DIPB(),   
      .DOB(Pattern4[15:0]),    .DOPB(Pattern4[17:16])            );
 RAMB16_S18_S18 PatternRAM5 (
      .CLKA(clk),     .ENA(1'b1),    .WEA(PatternsWrEn[5]),   // PORT A 
      .SSRA(Hard_rst),              // Synchronous Set/Reset Input
      .ADDRA(PatternsAddrA),  
      .DIA(D_IN[15:0]),            .DIPA(D_IN[17:16]),   
      .DOA(Pattern5Back[15:0]),    .DOPA(Pattern5Back[17:16]),   
      .CLKB(clk2x),   .ENB(PatternSel),    .WEB(1'b0),        // PORT B 
      .SSRB(Hard_rst),              // Synchronous Set/Reset Input
      .ADDRB(PatternsAddrB),   .DIB(),       .DIPB(),   
      .DOB(Pattern5[15:0]),    .DOPB(Pattern5[17:16])            );
 RAMB16_S18_S18 PatternRAM6 (
      .CLKA(clk),     .ENA(1'b1),    .WEA(PatternsWrEn[6]),   // PORT A 
      .SSRA(Hard_rst),              // Synchronous Set/Reset Input
      .ADDRA(PatternsAddrA),  
      .DIA(D_IN[15:0]),            .DIPA(D_IN[17:16]),   
      .DOA(Pattern6Back[15:0]),    .DOPA(Pattern6Back[17:16]),   
      .CLKB(clk2x),   .ENB(PatternSel),    .WEB(1'b0),        // PORT B 
      .SSRB(Hard_rst),              // Synchronous Set/Reset Input
      .ADDRB(PatternsAddrB),   .DIB(),       .DIPB(),   
      .DOB(Pattern6[15:0]),    .DOPB(Pattern6[17:16])            );
 RAMB16_S18_S18 PatternRAM7 (
      .CLKA(clk),     .ENA(1'b1),    .WEA(PatternsWrEn[7]),   // PORT A 
      .SSRA(Hard_rst),              // Synchronous Set/Reset Input
      .ADDRA(PatternsAddrA),  
      .DIA(D_IN[15:0]),            .DIPA(D_IN[17:16]),   
      .DOA(Pattern7Back[15:0]),    .DOPA(Pattern7Back[17:16]),   
      .CLKB(clk2x),   .ENB(PatternSel),    .WEB(1'b0),        // PORT B 
      .SSRB(Hard_rst),              // Synchronous Set/Reset Input
      .ADDRB(PatternsAddrB),   .DIB(),       .DIPB(),   
      .DOB(Pattern7[15:0]),    .DOPB(Pattern7[17:16])            );
 RAMB16_S18_S18 PatternRAM8 (
      .CLKA(clk),     .ENA(1'b1),    .WEA(PatternsWrEn[8]),   // PORT A 
      .SSRA(Hard_rst),              // Synchronous Set/Reset Input
      .ADDRA(PatternsAddrA),  
      .DIA(D_IN[15:0]),            .DIPA(D_IN[17:16]),   
      .DOA(Pattern8Back[15:0]),    .DOPA(Pattern8Back[17:16]),   
      .CLKB(clk2x),   .ENB(PatternSel),    .WEB(1'b0),        // PORT B 
      .SSRB(Hard_rst),              // Synchronous Set/Reset Input
      .ADDRB(PatternsAddrB),   .DIB(),       .DIPB(),   
      .DOB(Pattern8[15:0]),    .DOPB(Pattern8[17:16])            );

// **** 8 Input FIFOs (1 per fiber) & 1 output (data to DCC) ***

fifo_ic_v2  InSpyFIFO1 
 ( .fifo_gsr_in(Hard_rst),            .write_data_in({18'b0,InputSpyData1}), 
   .write_clock_in(clk2x),            .write_enable_in(~StartStop_Spy),
   .read_clock_in(clk),               .read_enable_in(InSpyFIFO1_RD),
   .read_data_out({pad18_1, InSpyFIFO1_out}), .fifostatus_out(),
   .full_out(InSpyFIFO_full[1]),      .empty_out(InSpyFIFO_empty[1])  );

fifo_ic_v2  InSpyFIFO2 
 ( .fifo_gsr_in(Hard_rst),            .write_data_in({18'b0,InputSpyData2}), 
   .write_clock_in(clk2x),            .write_enable_in(~StartStop_Spy),
   .read_clock_in(clk),               .read_enable_in(InSpyFIFO2_RD),
   .read_data_out({pad18_2,InSpyFIFO2_out}), .fifostatus_out(),
   .full_out(InSpyFIFO_full[2]),      .empty_out(InSpyFIFO_empty[2])  );

fifo_ic_v2  InSpyFIFO3 
 ( .fifo_gsr_in(Hard_rst),            .write_data_in({18'b0,InputSpyData3}), 
   .write_clock_in(clk2x),            .write_enable_in(~StartStop_Spy),
   .read_clock_in(clk),               .read_enable_in(InSpyFIFO3_RD),
   .read_data_out({pad18_3,InSpyFIFO3_out}), .fifostatus_out(),
   .full_out(InSpyFIFO_full[3]),      .empty_out(InSpyFIFO_empty[3])  );

fifo_ic_v2  InSpyFIFO4 
 ( .fifo_gsr_in(Hard_rst),            .write_data_in({18'b0,InputSpyData4}), 
   .write_clock_in(clk2x),            .write_enable_in(~StartStop_Spy),
   .read_clock_in(clk),               .read_enable_in(InSpyFIFO4_RD),
   .read_data_out({pad18_4,InSpyFIFO4_out}), .fifostatus_out(),
   .full_out(InSpyFIFO_full[4]),      .empty_out(InSpyFIFO_empty[4])  );

fifo_ic_v2  InSpyFIFO5 
 ( .fifo_gsr_in(Hard_rst),            .write_data_in({18'b0,InputSpyData5}), 
   .write_clock_in(clk2x),            .write_enable_in(~StartStop_Spy),
   .read_clock_in(clk),               .read_enable_in(InSpyFIFO5_RD),
   .read_data_out({pad18_5,InSpyFIFO5_out}), .fifostatus_out(),
   .full_out(InSpyFIFO_full[5]),      .empty_out(InSpyFIFO_empty[5])  );

fifo_ic_v2  InSpyFIFO6 
 ( .fifo_gsr_in(Hard_rst),            .write_data_in({18'b0,InputSpyData6}), 
   .write_clock_in(clk2x),            .write_enable_in(~StartStop_Spy),
   .read_clock_in(clk),               .read_enable_in(InSpyFIFO6_RD),
   .read_data_out({pad18_6,InSpyFIFO6_out}), .fifostatus_out(),
   .full_out(InSpyFIFO_full[6]),      .empty_out(InSpyFIFO_empty[6])  );

fifo_ic_v2  InSpyFIFO7 
 ( .fifo_gsr_in(Hard_rst),            .write_data_in({18'b0,InputSpyData7}), 
   .write_clock_in(clk2x),            .write_enable_in(~StartStop_Spy),
   .read_clock_in(clk),               .read_enable_in(InSpyFIFO7_RD),
   .read_data_out({pad18_7,InSpyFIFO7_out}), .fifostatus_out(),
   .full_out(InSpyFIFO_full[7]),      .empty_out(InSpyFIFO_empty[7])  );

fifo_ic_v2  InSpyFIFO8 
 ( .fifo_gsr_in(Hard_rst),            .write_data_in({18'b0,InputSpyData8}), 
   .write_clock_in(clk2x),            .write_enable_in(~StartStop_Spy),
   .read_clock_in(clk),               .read_enable_in(InSpyFIFO8_RD),
   .read_data_out({pad18_8,InSpyFIFO8_out}), .fifostatus_out(),
   .full_out(InSpyFIFO_full[8]),      .empty_out(InSpyFIFO_empty[8])  );
//
// this fifo is for testing the linearization....can be deleted later (8/05)
//

fifo_ic_2047x18  ETSpyFIFO 
  ( .fifo_gsr_in(Soft_rst),
   .write_data_in({8'h0, ET}), 
   .write_clock_in(clk),            .write_enable_in(~StartStop_Spy),
   .read_clock_in(clk),             .read_enable_in(ETSpyFIFO_RD), 
   .read_data_out(ETSpy),    .fifostatus_out(),
   .full_out(ETSpy_full),        .empty_out(ETSpy_empty)  );

//  Snapshot FIFOs to spy around a Link_error occurence
snapshot snapFifo1 (
   .Click(InputSpyData1[17]), 
   .DLL_lock(DLL_lock),  .TI_EN(TI_EN[1]),  .TTC_Ready(TTC_Ready), 
   .clk(clk),   .clk2x(clk2x),    .rst(Hard_rst),    .RdEn(SnapRdEn[1]), 
   .InputSpyData(InputSpyData1),    .BCN(BCN), 
   .empty(SnapFifoEmpty[1]),  .full(), .SnapFIFO_out(SnapFIFO1_out)  );

snapshot snapFifo2 (
   .Click(InputSpyData2[17]), 
   .DLL_lock(DLL_lock),  .TI_EN(TI_EN[2]),  .TTC_Ready(TTC_Ready), 
   .clk(clk),   .clk2x(clk2x),    .rst(Hard_rst),    .RdEn(SnapRdEn[2]), 
   .InputSpyData(InputSpyData2),    .BCN(BCN), 
   .empty(SnapFifoEmpty[2]),  .full(), .SnapFIFO_out(SnapFIFO2_out)  );

snapshot snapFifo3 (
   .Click(InputSpyData3[17]), 
   .DLL_lock(DLL_lock),  .TI_EN(TI_EN[3]),  .TTC_Ready(TTC_Ready), 
   .clk(clk),   .clk2x(clk2x),    .rst(Hard_rst),    .RdEn(SnapRdEn[3]), 
   .InputSpyData(InputSpyData3),    .BCN(BCN), 
   .empty(SnapFifoEmpty[3]),  .full(), .SnapFIFO_out(SnapFIFO3_out)  );

snapshot snapFifo4 (
   .Click(InputSpyData4[17]), 
   .DLL_lock(DLL_lock),  .TI_EN(TI_EN[4]),  .TTC_Ready(TTC_Ready), 
   .clk(clk),   .clk2x(clk2x),    .rst(Hard_rst),    .RdEn(SnapRdEn[4]), 
   .InputSpyData(InputSpyData4),    .BCN(BCN), 
   .empty(SnapFifoEmpty[4]),  .full(), .SnapFIFO_out(SnapFIFO4_out)  );

snapshot snapFifo5 (
   .Click(InputSpyData5[17]), 
   .DLL_lock(DLL_lock),  .TI_EN(TI_EN[5]),  .TTC_Ready(TTC_Ready), 
   .clk(clk),   .clk2x(clk2x),    .rst(Hard_rst),    .RdEn(SnapRdEn[5]), 
   .InputSpyData(InputSpyData5),    .BCN(BCN), 
   .empty(SnapFifoEmpty[5]),  .full(), .SnapFIFO_out(SnapFIFO5_out)  );

snapshot snapFifo6 (
   .Click(InputSpyData6[17]), 
   .DLL_lock(DLL_lock),  .TI_EN(TI_EN[6]),  .TTC_Ready(TTC_Ready), 
   .clk(clk),   .clk2x(clk2x),    .rst(Hard_rst),    .RdEn(SnapRdEn[6]), 
   .InputSpyData(InputSpyData6),    .BCN(BCN), 
   .empty(SnapFifoEmpty[6]),  .full(), .SnapFIFO_out(SnapFIFO6_out)  );

snapshot snapFifo7 (
   .Click(InputSpyData7[17]), 
   .DLL_lock(DLL_lock),  .TI_EN(TI_EN[7]),  .TTC_Ready(TTC_Ready), 
   .clk(clk),   .clk2x(clk2x),    .rst(Hard_rst),    .RdEn(SnapRdEn[7]), 
   .InputSpyData(InputSpyData7),    .BCN(BCN), 
   .empty(SnapFifoEmpty[7]),  .full(), .SnapFIFO_out(SnapFIFO7_out)  );

snapshot snapFifo8 (
   .Click(InputSpyData8[17]), 
   .DLL_lock(DLL_lock),  .TI_EN(TI_EN[8]),  .TTC_Ready(TTC_Ready), 
   .clk(clk),   .clk2x(clk2x),    .rst(Hard_rst),    .RdEn(SnapRdEn[8]), 
   .InputSpyData(InputSpyData8),    .BCN(BCN), 
   .empty(SnapFifoEmpty[8]),  .full(), .SnapFIFO_out(SnapFIFO8_out)  );

 // control logic for L2OutSpyFIFO: the FIFO records event-blocks until full,
 // then is de-activated; it's reactivaded once emptyied.
 // Reading the FIFO, all words part of an event-block come 
 // with L2OutSpyFIFOempty=0. 
 assign HEADER  = (L2StatusBits[1:0]==2'b11);
 assign TRAILER = (L2StatusBits[1:0]==2'b01);
 always @ (posedge clk)
  if (Soft_rst)    L2WrEn <= 1'b0;  
  else  begin
   TRAILER_d      <= TRAILER;
   TRAILER_2d     <= TRAILER_d;
   L2Data_d       <= L2Data;       // to align with L2WrEn
   L2StatusBits_d <= L2StatusBits; // to align with L2WrEn
   if (L2OutSpyFIFOempty & ~L2OutSpyFIFOfull)  L2OutSpyFIFOactive <= 1;
   if (L2OutSpyFIFOfull  & ~L2OutSpyFIFOempty) L2OutSpyFIFOactive <= 0;
   if (HEADER & L2OutSpyFIFOactive)  L2WrEn <= 1;
   else if (TRAILER_2d)              L2WrEn <= 0; 
 end

fifo_ic_2047x18  L2OutSpyFIFO 
  ( .fifo_gsr_in(Soft_rst),
   .write_data_in({L2StatusBits_d[1:0], L2Data_d}), 
   .write_clock_in(clk),            .write_enable_in(L2WrEn),
   .read_clock_in(clk),             .read_enable_in(L2OutSpyFIFO_RD), 
   .read_data_out(L2OutSpyFIFO_out),    .fifostatus_out(),
   .full_out(L2OutSpyFIFOfull),        .empty_out(L2OutSpyFIFOempty)  );
        
endmodule