/*************************************************************************
DESCRIPTION :
The TTCsynchronizer crosses the clock domanins from TTC_CK40Des1 to clk.
The input pulse must be longer than 12.5 ns for the circuit to work.
This covers the ample jitter of the TTC signals (25ns +/- jitter).
The resulting output signal is a 1-clk-tick pulses.
***************************************************************************/

`timescale 1ns / 1ns
module TTCsynchronizer  (

 input   clk, clk2x,   t_Command,  
 output  reg  Command = 0      );

 reg [5:1] Command_D = 0; //try killing powerup glitches with "= 0"
 reg Command_clk2x = 0;
 
 always @ (posedge clk2x)    // digital edge-detection of t_Command
   begin 
     Command_D[1]     <= t_Command;
     Command_D[2]    <= Command_D[1];
     Command_D[3]    <= Command_D[2];
     Command_clk2x <= Command_D[1] | Command_D[2] | Command_D[3];// ultra-conservative
   end

 always @ (posedge clk )   begin   
    // detect transition 0->1 and make a 25-ns pulse
    Command_D[4] <= Command_clk2x;    
    Command_D[5] <= Command_D[4];  
    Command    <= Command_D[4]  & ~Command_D[5];   
  end  
    
endmodule  