/****************************************************************
MODULE  :  snapshot        AUTHOR    :  Tullio

DESCRIPTION :
It takes snapshot whenever the Click signal is "1".
The snapshot is a sequence of 7 Data words centered around
the Click pulse. An odd number ("7") helps to distinguish
subsequent snapshots.
If a following pulse on the Click signal happens less than
10 ticks after, is ignored.
*****************************************************************/
 
`timescale 1ns / 1ns
module snapshot
  ( Click,    InputSpyData,
	 DLL_lock, TI_EN,   TTC_Ready,  BCN,    
    empty, full, SnapFIFO_out,
    clk, clk2x, rst, RdEn    	     	);
 
// ***** PARAMETERS declaration on this module... 
  parameter
  BCN_WIDTH                   = 12, 
  InSpyFIFO_WIDTH             = 18;

 input  Click, DLL_lock, TI_EN, TTC_Ready;
 input  clk, clk2x, rst, RdEn;
 input  [InSpyFIFO_WIDTH-1:0] InputSpyData;
 input  [BCN_WIDTH-1:0] BCN;   

 output  empty,	full;
 output  [35:0] SnapFIFO_out;
	
//  ***  INTERNAL SIGNALS ***
  reg  [InSpyFIFO_WIDTH-1:0] D1, D2, D3, D4, D5, D6, D7;
  reg  WrEn, FIFOactive;
  reg  C2, C3, C4, C5, C6, C7, C8, C9, C10, C11, C12, C13, C14, C15, EOS;

 // control logic for FIFO: the FIFO records snapshots until full,
 // then is de-activated; it's reactivaded once emptyied.
 always @ (posedge clk2x)
  if (rst)    begin
    WrEn <= 1'b0;  
   end
  else  begin
    D7 <= D6; D6 <= D5; D5 <= D4; D4 <= D3;
    D3 <= D2; D2<= D1; D1 <= InputSpyData; // center the data
    EOS <= C15; C15 <= C14; C14 <= C13; C13 <= C12; C12 <= C11; 
    C11 <= C10; C10 <= C9; C9 <= C8; C8 <= C7; C7 <= C6; C6 <= C5;
    C5  <= C4; C4 <= C3; C3 <= C2; C2 <= Click;
    if (empty & ~full)  FIFOactive <= 1;
    if (full  & ~empty) FIFOactive <= 0;
    if (Click & FIFOactive)  WrEn <= 1;
    else if (EOS)            WrEn <= 0;  // End Of Snapshot
 end

fifo_ic_v2   SnapFIFO 
  ( .fifo_gsr_in(rst),
    .write_data_in( {3'b0, BCN, DLL_lock, TI_EN, TTC_Ready, D7} ), 
    .write_clock_in(clk2x),              .write_enable_in(WrEn),
    .read_clock_in(clk),                 .read_enable_in(RdEn), 
    .read_data_out(SnapFIFO_out),        .fifostatus_out(),
    .full_out(full),                     .empty_out(empty)  );
				
endmodule