// One channel of the Trigger Path

module TpgSingleChannel
 ( 
input  [6:0]  QIEData,	 //the incoming data from the input stage
input  CLK,    reset_TP,
input  bypass, // when "1" disable EnFilter for data and MuonBit
input  [6:0]  INPUT_LUT_INIT_ADDR,  // the input LUT 
input  InLutWE, InLUT_Mode, 
input  [10:0] INPUT_LUT_INIT_DATA, 
output [10:0] InLutDOut,
input  [9:0]  COMPRESSION_LUT_INIT_ADDR,   // the output LUT
input  ComprLutWE, OutLUT_Mode, 
input  [7:0]  COMPRESSION_LUT_INIT_DATA, 
output [7:0]  OutLUT_DOut,
output reg [9:0] ETvalue,
output [8:0]  TP         );

wire  [9:0]  Peak,  input_compr_LUT;
wire  PeakDetected,  MuonBit;
reg   [10:0] InLutDOut_D, InLutDOut_2D, InLutDOut_3D;

assign input_compr_LUT = bypass ? InLutDOut_2D[9:0] : Peak;
always @ (posedge CLK) ETvalue <= input_compr_LUT;
assign MuonBit = (PeakDetected | bypass) & InLutDOut_3D[10];
assign TP = reset_TP ? 'd0: {MuonBit, OutLUT_DOut};

lut     InLUT(      // linearizes + Et + muon
    .DIN(QIEData),               .INIT_ADDR(INPUT_LUT_INIT_ADDR), 
    .LUT_WRITE(InLutWE),         .INIT_DATA(INPUT_LUT_INIT_DATA),
    .LUT_MODE(InLUT_Mode),       .CLK(CLK),       .DOUT(InLutDOut)    );

EnFilterRev2   EnFilter (
    .clk(CLK),            .DataIn(InLutDOut[9:0]), 
    .Peak(Peak),          .PeakDetected(PeakDetected)   );

 always@(posedge CLK) begin  // compensate EnFilter delay
   InLutDOut_D  <= InLutDOut;
   InLutDOut_2D <= InLutDOut_D;
   InLutDOut_3D <= InLutDOut_2D;
 end

output_LUT_Bram ComprLUT (  //this module performs the compression 
    .DIN(input_compr_LUT), 
    .INIT_ADDR(COMPRESSION_LUT_INIT_ADDR), 
    .LUT_WRITE(ComprLutWE), 
    .INIT_DATA(COMPRESSION_LUT_INIT_DATA), 
    .LUT_MODE(OutLUT_Mode),
    .CLK(CLK),        .DOUT(OutLUT_DOut)          );

endmodule